@@ -1,6 +1,7 @@
 import logging
 
 from datetime import datetime
+from math import copysign
 from jal.constants import Setup, BookAccount, TransactionType, ActionSubtype, TransferSubtype, CorporateAction, \
     PredefinedCategory, PredefinedPeer
 from PySide2.QtCore import Qt, QDate, QDateTime
@@ -81,6 +82,8 @@ class Ledger:
             self.updateBalancesView()
 
     def updateBalancesView(self):
+        if self.balances_view is None:
+            return
         calculateBalances(self.db, self.balance_date, self.balance_currency, self.balance_active_only)
         self.balances_view.model().select()
 
@@ -98,6 +101,8 @@ class Ledger:
             self.updateHoldingsView()
 
     def updateHoldingsView(self):
+        if self.holdings_view is None:
+            return
         calculateHoldings(self.db, self.holdings_date, self.holdings_currency)
         holdings_model = self.holdings_view.model()
         holdings_model.select()
@@ -251,159 +256,108 @@ class Ledger:
             self.current['category'] = PredefinedCategory.Taxes
             self.appendTransaction(BookAccount.Costs, tax_amount)
 
-    def processBuy(self):
-        seq_id = self.current_seq
-        account_id = self.current['account']
-        asset_id = self.current['asset']
-        qty = self.current['amount']
-        price = self.current['price']
-        trade_value = round(price * qty, 2) + self.current['fee_tax'] + self.current['coupon']
-        sell_qty = 0
-        sell_value = 0
-        if self.getAmount(BookAccount.Assets, asset_id) < 0:        # Match deals if we have something sold before
-            # Get information about last deal
-            query = executeSQL(self.db,
-                               "SELECT d.open_sid AS open, ABS(o.qty) - SUM(d.qty) AS remainder FROM deals AS d "
-                               "LEFT JOIN sequence AS os ON os.type=3 AND os.id=d.open_sid "
-                               "JOIN trades AS o ON o.id = os.operation_id "
-                               "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
-                               "GROUP BY d.open_sid "
-                               "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
-                               [(":account_id", account_id), (":asset_id", asset_id)])
-            last_sid = 0  # Take all Sell operations by default
-            reminder = 0
-            if query.next():
-                # Get sid of Sell trade from the last deal and non-matched reminder of last Sell trade
-                last_sid, reminder = readSQLrecord(query)
-                # if last Sell is fully matched (reminder>=0) we start from next trade, otherwise we need to shift by 1
-                if reminder < 0:
-                    last_sid = last_sid - 1
-            query = executeSQL(self.db,
-                               "SELECT s.id, -t.qty, t.price FROM trades AS t "
-                               "LEFT JOIN sequence AS s ON s.type = 3 AND s.operation_id=t.id "
-                               "WHERE t.qty < 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
-                               "AND s.id < :sid AND s.id > :last_sid",
-                               [(":asset_id", asset_id), (":account_id", account_id),
-                                (":sid", seq_id), (":last_sid", last_sid)])
-            while query.next():   # Perform match ("closure") of previous Sell trades
-                deal_sid, deal_qty, deal_price = readSQLrecord(query)
-                if reminder < 0:
-                    next_deal_qty = reminder
-                    reminder = 0
-                else:
-                    next_deal_qty = deal_qty
-                if (sell_qty + next_deal_qty) >= qty:  # we are buying less or the same amount as was sold previously
-                    next_deal_qty = qty - sell_qty
-                _ = executeSQL(self.db, "INSERT INTO deals(account_id, asset_id, open_sid, close_sid, qty) "
-                                        "VALUES(:account_id, :asset_id, :open_sid, :close_sid, :qty)",
-                               [(":account_id", account_id), (":asset_id", asset_id), (":open_sid", deal_sid),
-                                (":close_sid", seq_id), (":qty", -next_deal_qty)])
-                sell_qty = sell_qty + next_deal_qty
-                sell_value = sell_value + (next_deal_qty * deal_price)
-                if sell_qty == qty:
-                    break
-        credit_taken = self.takeCredit(trade_value)
-        if trade_value != credit_taken:
-            self.appendTransaction(BookAccount.Money, -(trade_value - credit_taken))
-        if sell_qty > 0:  # Add result of closed deals
-            self.appendTransaction(BookAccount.Assets, sell_qty, sell_value)
-            self.current['category'] = PredefinedCategory.Profit
-            self.appendTransaction(BookAccount.Incomes, ((price * sell_qty) - sell_value))
-        if sell_qty < qty:  # Add new long position
-            self.appendTransaction(BookAccount.Assets, (qty - sell_qty), (qty - sell_qty) * price)
-        if self.current['coupon']:
-            self.current['category'] = PredefinedCategory.Dividends
-            self.appendTransaction(BookAccount.Costs, self.current['coupon'])
-        if self.current['fee_tax']:
-            self.current['category'] = PredefinedCategory.Fees
-            self.appendTransaction(BookAccount.Costs, self.current['fee_tax'])
-
-    def processSell(self):
-        seq_id = self.current_seq
-        account_id = self.current['account']
-        asset_id = self.current['asset']
-        qty = -self.current['amount']
-        price = self.current['price']
-        trade_value = round(price * qty, 2) - self.current['fee_tax'] + self.current['coupon']
-        buy_qty = 0
-        buy_value = 0
-        if self.getAmount(BookAccount.Assets, asset_id) > 0:    # Match deals if we have something bought before
-            # Get information about last deal
-            query = executeSQL(self.db,
-                               "SELECT d.open_sid AS open, ABS(coalesce(o.qty, ca.qty_new))- SUM(d.qty) AS remainder "
-                               "FROM deals AS d "
-                               "LEFT JOIN sequence AS os ON os.id=d.open_sid "
-                               "LEFT JOIN trades AS o ON o.id = os.operation_id AND os.type = 3 "
-                               "LEFT JOIN corp_actions AS ca ON ca.id = os.operation_id AND os.type = 5 "
-                               "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
-                               "GROUP BY d.open_sid "
-                               "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
-                               [(":account_id", account_id), (":asset_id", asset_id)])
-            last_sid = 0  # Take all Buy operations by default
-            reminder = 0
-            if query.next():     # Perform match ("closure") of previous Buy trades
-                # Get sid of Buy trade from the last deal and non-matched reminder of last Buy trade
-                last_sid, reminder = readSQLrecord(query)
-                # if last Buy is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
-                if reminder > 0:
-                    last_sid = last_sid - 1
-            query = executeSQL(self.db,
-                               "SELECT * FROM ("
-                               "SELECT s.id, t.qty, t.price FROM trades AS t "
-                               "LEFT JOIN sequence AS s ON s.type = 3 AND s.operation_id=t.id "
-                               "WHERE t.qty > 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
-                               "AND s.id < :sid AND s.id > :last_sid "
-                               "UNION ALL "
-                               "SELECT s.id, c.qty_new, coalesce(l.value/c.qty_new, 0) AS price FROM corp_actions AS c "
-                               "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id " 
-                               "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id_new AND l.value > 0 "
-                               "WHERE c.qty_new > 0 AND c.asset_id_new = :asset_id AND c.account_id = :account_id "
-                               "AND s.id < :sid AND s.id > :last_sid " 
-                               ")ORDER BY id",
-                               [(":asset_id", asset_id), (":account_id", account_id),
-                                (":sid", seq_id), (":last_sid", last_sid)])
-            while query.next():    # Perform match ("closure") of previous Buy trades
-                deal_sid, deal_qty, deal_price = readSQLrecord(query)  # deal_sid -> trade_sid
-                if reminder > 0:
-                    next_deal_qty = reminder
-                    reminder = 0
-                else:
-                    next_deal_qty = deal_qty
-                if (buy_qty + next_deal_qty) >= qty:  # we are selling less or the same amount as was bought previously
-                    next_deal_qty = qty - buy_qty
-                _ = executeSQL(self.db, "INSERT INTO deals(account_id, asset_id, open_sid, close_sid, qty) "
-                                        "VALUES(:account_id, :asset_id, :open_sid, :close_sid, :qty)",
-                               [(":account_id", account_id), (":asset_id", asset_id), (":open_sid", deal_sid),
-                                (":close_sid", seq_id), (":qty", next_deal_qty)])
-                buy_qty = buy_qty + next_deal_qty
-                buy_value = buy_value + (next_deal_qty * deal_price)
-                if buy_qty == qty:
-                    break
-        credit_returned = self.returnCredit(trade_value)
-        if credit_returned < trade_value:
-            self.appendTransaction(BookAccount.Money, (trade_value - credit_returned))
-        if buy_qty > 0:  # Add result of closed deals
-            self.appendTransaction(BookAccount.Assets, -buy_qty, -buy_value)
-            self.current['category'] = PredefinedCategory.Profit
-            self.appendTransaction(BookAccount.Incomes, (buy_value - (price * buy_qty)))
-        if buy_qty < qty:  # Add new short position
-            self.appendTransaction(BookAccount.Assets, (buy_qty - qty), (buy_qty - qty) * price)
-        if self.current['coupon']:
-            self.current['category'] = PredefinedCategory.Dividends
-            self.appendTransaction(BookAccount.Incomes, -self.current['coupon'])
-        if self.current['fee_tax']:
-            self.current['category'] = PredefinedCategory.Fees
-            self.appendTransaction(BookAccount.Costs, self.current['fee_tax'])
-
+    # Process buy or sell operation base on self.current['amount'] (>0 - buy, <0 - sell)
     def processTrade(self):
         if self.current['peer'] == '':
             logging.error(g_tr('Ledger', "Can't process trade as bank isn't set for investment account"))
             return
-        operationTrade = {
-            -1: self.processSell,
-            1: self.processBuy
-        }
-        operationTrade[self.current['subtype']]()
+
+        seq_id = self.current_seq
+        account_id = self.current['account']
+        asset_id = self.current['asset']
+        type = copysign(1, self.current['amount'])  # 1 is buy, -1 is sell
+        qty = type * self.current['amount']
+        price = self.current['price']
+
+        trade_value = round(price * qty, 2) + type * self.current['fee_tax'] + self.current['coupon']
+
+        processed_qty = 0
+        processed_value = 0
+        # Get asset amount accumulated before current operation
+        asset_amount = self.getAmount(BookAccount.Assets, asset_id)
+        if ((-type) * asset_amount) > 0:  # Process deal match if we have asset that is opposite to operation
+            # Get information about last deal with quantity of opposite sign
+            last_sid = readSQL(self.db, "SELECT "
+                                        "CASE WHEN (:type)*qty<0 THEN open_sid ELSE close_sid END AS last_sid "
+                                        "FROM deals "
+                                        "WHERE account_id=:account_id AND asset_id=:asset_id "
+                                        "ORDER BY close_sid DESC, open_sid DESC LIMIT 1",
+                               [(":type", type), (":account_id", account_id), (":asset_id", asset_id)])
+            last_sid = 0 if last_sid is None else last_sid
+            # Next get information about abs trade quantity that was in this last deal
+            last_qty = readSQL(self.db, "SELECT coalesce(ABS(t.qty), 0)+coalesce(ABS(ca.qty_new) , 0) AS qty "
+                                        "FROM sequence AS s "
+                                        "LEFT JOIN trades AS t ON t.id=s.operation_id AND s.type=3 "
+                                        "LEFT JOIN corp_actions AS ca ON ca.id=s.operation_id AND s.type=5 "
+                                        "WHERE s.id=:last_sid", [(":last_sid", last_sid)])
+            last_qty = 0 if last_qty is None else last_qty
+            # Collect quantity of all deals where this last opposite trade participated (positive value)
+            deals_qty = readSQL(self.db, "SELECT coalesce(SUM(ABS(qty)), 0) "
+                                         "FROM deals AS d "
+                                         "WHERE account_id=:account_id AND asset_id=:asset_id "
+                                         "AND (open_sid=:last_sid OR close_sid=:last_sid)",
+                                [(":account_id", account_id), (":asset_id", asset_id), (":last_sid", last_sid)])
+            reminder = last_qty - deals_qty
+            # if last trade is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
+            if reminder > 0:
+                last_sid = last_sid - 1
+
+            # Get a list of all previous not matched trades or corporate actions of opposite direction (type parameter)
+            query = executeSQL(self.db,
+                               "SELECT * FROM ("
+                               "SELECT s.id, ABS(t.qty), t.price FROM trades AS t "
+                               "LEFT JOIN sequence AS s ON s.type = 3 AND s.operation_id=t.id "
+                               "WHERE (:type)*qty < 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
+                               "AND s.id < :sid AND s.id > :last_sid "
+                               "UNION ALL "
+                               "SELECT s.id, ABS(c.qty_new), coalesce(l.value/c.qty_new, 0) AS price FROM corp_actions AS c "
+                               "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
+                               "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id_new AND l.value > 0 "
+                               "WHERE (:type)*c.qty_new < 0 AND c.asset_id_new=:asset_id AND c.account_id=:account_id "
+                               "AND s.id < :sid AND s.id > :last_sid "
+                               ")ORDER BY id",
+                               [(":type", type), (":asset_id", asset_id), (":account_id", account_id),
+                                (":sid", seq_id), (":last_sid", last_sid)])
+            while query.next():  # Perform match ("closure") of previous trades
+                deal_sid, deal_qty, deal_price = readSQLrecord(query)    # deal_sid -> trade_sid
+                if reminder > 0:
+                    next_deal_qty = reminder
+                    reminder = 0
+                else:
+                    next_deal_qty = deal_qty
+                if (processed_qty + next_deal_qty) >= qty:  # We can't process more than qty of current trade
+                    next_deal_qty = qty - processed_qty     # If it happens - just process the remainder of the trade
+                # Create a deal with relevant sign of quantity (-1 for short, +1 for long)
+                _ = executeSQL(self.db, "INSERT INTO deals(account_id, asset_id, open_sid, close_sid, qty) "
+                                        "VALUES(:account_id, :asset_id, :open_sid, :close_sid, :qty)",
+                               [(":account_id", account_id), (":asset_id", asset_id), (":open_sid", deal_sid),
+                                (":close_sid", seq_id), (":qty", (-type)*next_deal_qty)])
+                processed_qty += next_deal_qty
+                processed_value += (next_deal_qty * deal_price)
+                if processed_qty == qty:
+                    break
+        if type > 0:
+            credit_value = self.takeCredit(trade_value)
+        else:
+            credit_value = self.returnCredit(trade_value)
+        if credit_value < trade_value:
+            self.appendTransaction(BookAccount.Money, (-type)*(trade_value - credit_value))
+        if processed_qty > 0:  # Add result of closed deals
+            # decrease (for sell) or increase (for buy) amount of assets in ledger
+            self.appendTransaction(BookAccount.Assets, type*processed_qty, type*processed_value)
+            self.current['category'] = PredefinedCategory.Profit
+            self.appendTransaction(BookAccount.Incomes, type * ((price * processed_qty) - processed_value))
+        if processed_qty < qty:  # We have reminder that opens a new position
+            self.appendTransaction(BookAccount.Assets, type*(qty - processed_qty), type*(qty - processed_qty) * price)
+        if self.current['coupon']:
+            self.current['category'] = PredefinedCategory.Dividends
+            if type> 0:
+                self.appendTransaction(BookAccount.Costs, self.current['coupon'])
+            else:
+                self.appendTransaction(BookAccount.Incomes, type*self.current['coupon'])
+        if self.current['fee_tax']:
+            self.current['category'] = PredefinedCategory.Fees
+            self.appendTransaction(BookAccount.Costs, self.current['fee_tax'])
 
     def processTransferOut(self):
         amount = -self.current['amount']
@@ -513,7 +467,7 @@ class Ledger:
         self.current['peer'] = None
         self.current['fee_tax'] = 0
         self.current['tag'] = 0
-        self.processBuy()
+        self.processTrade()
 
     def processCorporateAction(self):
         operationCorpAction = {
