@@ -276,8 +276,8 @@ class TableViewConfig:
             (parent.action_Load_quotes,     "triggered()",              partial(parent.downloader.showQuoteDownloadDialog, parent)),
             (parent.actionImportStatement,  "triggered()",              parent.statements.loadReport),
             (parent.actionImportSlipRU,     "triggered()",              parent.importSlip),
-            (parent.actionBackup,           "triggered()",              parent.Backup),
-            (parent.actionRestore,          "triggered()",              parent.Restore),
+            (parent.actionBackup,           "triggered()",              parent.backup.create),
+            (parent.actionRestore,          "triggered()",              parent.backup.restore),
             (parent.action_Re_build_Ledger, "triggered()",              partial(parent.ledger.showRebuildDialog, parent)),
             (parent.actionAccountTypes,     "triggered()",              partial(self.show_dialog, self.ACCOUNT_TYPES)),
             (parent.actionAccounts,         "triggered()",              partial(self.show_dialog, self.ACCOUNTS)),
